@@ -473,6 +473,7 @@ ensDbFromGRanges <- function(x, outfile, path, organism, genomeVersion, version)
     genes <- genes[ , c("gene_id", "gene_name", "entrezid", "gene_biotype",
                         "gene_seq_start", "gene_seq_end", "seq_name",
                         "seq_strand", "seq_coord_system")]
+    OK <- .checkIntegerCols(genes)
     dbWriteTable(con, name="gene", genes, overwrite=TRUE, row.names=FALSE)
     ## Done.
 
@@ -554,6 +555,7 @@ ensDbFromGRanges <- function(x, outfile, path, organism, genomeVersion, version)
     tx <- tx[ , c("tx_id", "tx_biotype", "tx_seq_start", "tx_seq_end",
                   "tx_cds_seq_start", "tx_cds_seq_end", "gene_id")]
     ## write the table.
+    OK <- .checkIntegerCols(tx)
     dbWriteTable(con, name="tx", tx, overwrite=TRUE, row.names=FALSE)
     rm(tx)
     rm(CDS)
@@ -584,6 +586,8 @@ ensDbFromGRanges <- function(x, outfile, path, organism, genomeVersion, version)
     exons <- unique(exons[ , c("exon_id", "start", "end")])
     colnames(exons) <- c("exon_id", "exon_seq_start", "exon_seq_end")
     ## writing the tables.
+    .checkIntegerCols(exons)
+    .checkIntegerCols(t2e)
     dbWriteTable(con, name="exon", exons, overwrite=TRUE, row.names=FALSE)
     dbWriteTable(con, name="tx2exon", t2e, overwrite=TRUE, row.names=FALSE)
     message("OK")
@@ -616,12 +620,7 @@ ensDbFromGRanges <- function(x, outfile, path, organism, genomeVersion, version)
     message("OK")
     message("Generating index...", appendLF=FALSE)
     ## generating all indices...
-    dbGetQuery(con, "create index seq_name_idx on chromosome (seq_name);")
-    dbGetQuery(con, "create index gene_id_idx on gene (gene_id);")
-    dbGetQuery(con, "create index tx_id_idx on tx (tx_id);")
-    dbGetQuery(con, "create index exon_id_idx on exon (exon_id);")
-    dbGetQuery(con, "create index t2e_tx_id_idx on tx2exon (tx_id);")
-    dbGetQuery(con, "create index t2e_exon_id_idx on tx2exon (exon_id);")
+    .createEnsDbIndices(con)
     message("OK")
     message("  -------------")
     message("Verifying validity of the information in the database:")
@@ -691,6 +690,7 @@ checkValidEnsDb <- function(x){
 ## organism is expected to be e.g. Homo_sapiens, so the full organism name, with
 ## _ as a separator
 tryGetSeqinfoFromEnsembl <- function(organism, ensemblVersion, seqnames){
+    ## Quick fix if organism contains whitespace instead of _:
     organism <- gsub(organism, pattern=" ", replacement="_", fixed=TRUE)
     Dataset <- paste0(c(tolower(.abbrevOrganismName(organism)), "gene_ensembl"),
                       collapse="_")
