@@ -5,26 +5,89 @@ DB <- EnsDb.Hsapiens.v75
 
 ## testing genes method.
 test_genes <- function(){
-    Gns <- genes(DB)
+    Gns <- genes(DB, filter=SeqnameFilter("Y"))
+    Gns <- genes(DB, filter=SeqnameFilter("Y"), return.type="DataFrame")
+    checkEquals(sort(colnames(Gns)), sort(listColumns(DB, "gene")))
+    Gns <- genes(DB, filter=SeqnameFilter("Y"), return.type="DataFrame",
+                 columns=c("gene_id", "tx_name"))
+    checkEquals(colnames(Gns), c("gene_id", "tx_name", "seq_name"))
+
+    Gns <- genes(DB, filter=SeqnameFilter("Y"), columns=c("gene_id", "gene_name"))
+    ## Here we don't need the seqnames in mcols!
+    checkEquals(colnames(mcols(Gns)), c("gene_id", "gene_name"))
+
+
     ## checkEquals(class(genes(DB, return.type="DataFrame",
     ##                         filter=list(SeqnameFilter("Y")))), "DataFrame" )
 }
 
 test_transcripts <- function(){
-    Tns <- transcripts(DB)
+    Tns <- transcripts(DB, filter=SeqnameFilter("Y"), return.type="DataFrame")
+    checkEquals(sort(colnames(Tns)), sort(c(listColumns(DB, "tx"), "seq_name")))
+
+    Tns <- transcripts(DB, columns=c("tx_id", "tx_name"), filter=SeqnameFilter("Y"))
+    checkEquals(sort(colnames(mcols(Tns))), sort(c("tx_id", "tx_name")))
+
+    ## Check the default ordering.
+    Tns <- transcripts(DB, filter = TxbiotypeFilter("protein_coding"),
+                       return.type = "data.frame",
+                       columns = c("seq_name", listColumns(DB, "tx")))
+    checkEquals(order(Tns$seq_name, method = "radix"), 1:nrow(Tns))
 }
 
 test_transcriptsBy <- function(){
-    TnsBy <- transcriptsBy(DB, filter=list(SeqnameFilter("Y")), by="gene")
+    ## Expect results on the forward strand to be ordered by tx_seq_start
+    res <- transcriptsBy(DB, filter = list(SeqnameFilter("Y"),
+                                           SeqstrandFilter("+")),
+                         by = "gene")
+    fw <- res[[3]]
+    checkEquals(order(start(fw)), 1:length(fw))
+    ## Expect results on the reverse strand to be ordered by -tx_seq_end
+    res <- transcriptsBy(DB, filter = list(SeqnameFilter("Y"),
+                                           SeqstrandFilter("-")),
+                         by = "gene")
+    rv <- res[[3]]
+    checkEquals(order(start(rv), decreasing = TRUE), 1:length(rv))
 }
 
 test_exons <- function(){
-    Exns <- exons(DB, filter=SeqnameFilter("X"))
-    Exns <- exons(DB, filter=list(SeqnameFilter("X")))
+    Exns <- exons(DB, filter=SeqnameFilter("Y"), return.type="DataFrame")
+    checkEquals(sort(colnames(Exns)), sort(c(listColumns(DB, "exon"), "seq_name")))
+
+    ## Check correct ordering.
+    Exns <- exons(DB, return.type = "data.frame", filter = SeqnameFilter(20:23))
+    checkEquals(order(Exns$seq_name, method = "radix"), 1:nrow(Exns))
 }
 
-test_exonsBy <- function(){
-    ExnsBy <- exonsBy(DB, filter=list(SeqnameFilter("X")), by="tx")
+test_exonsBy <- function() {
+    ##ExnsBy <- exonsBy(DB, filter=list(SeqnameFilter("X")), by="tx")
+    ExnsBy <- exonsBy(DB, filter = list(SeqnameFilter("Y")), by = "tx",
+                      columns = c("tx_name"))
+    checkEquals(sort(colnames(mcols(ExnsBy[[1]]))),
+                sort(c("exon_id", "exon_rank", "tx_name")))
+
+    ## Check what happens if we specify tx_id.
+    ExnsBy <- exonsBy(DB, filter=list(SeqnameFilter("Y")), by="tx",
+                      columns=c("tx_id"))
+    checkEquals(sort(colnames(mcols(ExnsBy[[1]]))),
+                sort(c("exon_id", "exon_rank", "tx_id")))
+
+    ## ExnsBy <- exonsBy(DB, filter=list(SeqnameFilter("Y")), by="tx",
+    ##                   columns=c("exon_rank"))
+    ## checkEquals(sort(colnames(mcols(ExnsBy[[1]]))),
+    ##             sort(c("exon_id", "exon_rank")))
+
+    ExnsBy <- exonsBy(DB, filter=list(SeqnameFilter("Y"), SeqstrandFilter("+")),
+                      by="gene")
+    ## Check that ordering is on start on the forward strand.
+    fw <- ExnsBy[[3]]
+    checkEquals(order(start(fw)), 1:length(fw))
+    ##
+    ExnsBy <- exonsBy(DB, filter=list(SeqnameFilter("Y"), SeqstrandFilter("-")),
+                      by="gene")
+    ## Check that ordering is on start on the forward strand.
+    rv <- ExnsBy[[3]]
+    checkEquals(order(end(rv), decreasing = TRUE), 1:length(rv))
 }
 
 test_dbfunctionality <- function(){
@@ -58,28 +121,45 @@ test_promoters <- function(){
 }
 
 test_return_columns_gene <- function(){
-    cols <- c("gene_name", "seq_name", "tx_id")
+    cols <- c("gene_name", "tx_id")
     Resu <- genes(DB, filter=SeqnameFilter("Y"), columns=cols, return.type="data.frame")
-    checkEquals(cols, colnames(Resu))
+    checkEquals(sort(c(cols, "seq_name", "gene_id")), sort(colnames(Resu)))
+
     Resu <- genes(DB, filter=SeqnameFilter("Y"), columns=cols, return.type="DataFrame")
-    checkEquals(cols, colnames(Resu))
+    checkEquals(sort(c(cols, "seq_name", "gene_id")), sort(colnames(Resu)))
+
+    Resu <- genes(DB, filter=SeqnameFilter("Y"), columns=cols)
+    checkEquals(sort(c(cols, "gene_id")), sort(colnames(mcols(Resu))))
 }
+
 test_return_columns_tx <- function(){
     cols <- c("tx_id", "exon_id", "tx_biotype")
     Resu <- transcripts(DB, filter=SeqnameFilter("Y"), columns=cols, return.type="data.frame")
-    checkEquals(cols, colnames(Resu))
+    checkEquals(sort(c(cols, "seq_name")), sort(colnames(Resu)))
+
     Resu <- transcripts(DB, filter=SeqnameFilter("Y"), columns=cols, return.type="DataFrame")
-    checkEquals(cols, colnames(Resu))
+    checkEquals(sort(c(cols, "seq_name")), sort(colnames(Resu)))
+
+    Resu <- transcripts(DB, filter=SeqnameFilter("Y"), columns=cols)
+    checkEquals(sort(cols), sort(colnames(mcols(Resu))))
 }
 test_return_columns_exon <- function(){
-    cols <- c("tx_id", "exon_id", "tx_biotype", "seq_name")
+    cols <- c("tx_id", "exon_id", "tx_biotype")
     Resu <- exons(DB, filter=SeqnameFilter("Y"), columns=cols, return.type="data.frame")
-    checkEquals(cols, colnames(Resu))
+    checkEquals(sort(c(cols, "seq_name")), sort(colnames(Resu)))
+
     Resu <- exons(DB, filter=SeqnameFilter("Y"), columns=cols, return.type="DataFrame")
-    checkEquals(cols, colnames(Resu))
+    checkEquals(sort(c(cols, "seq_name")), sort(colnames(Resu)))
+
+    Resu <- exons(DB, filter=SeqnameFilter("Y"), columns=cols)
+    checkEquals(sort(cols), sort(colnames(mcols(Resu))))
 }
 
 test_cdsBy <- function(){
+    ## Just checking if we get also tx_name
+    cs <- cdsBy(DB, filter=SeqnameFilter("Y"), column="tx_name")
+    checkTrue(any(colnames(mcols(cs[[1]])) == "tx_name"))
+
     do.plot <- FALSE
     ## By tx
     cs <- cdsBy(DB, filter=list(SeqnameFilter("Y"), SeqstrandFilter("+")))
@@ -146,30 +226,81 @@ test_cdsByGene <- function(){
                 by="gene", use.names=TRUE)
 }
 
-test_UTRs <- function(){
+test_UTRs <- function() {
+    ## check presence of tx_name
+    fUTRs <- fiveUTRsByTranscript(DB,
+                                  filter = TxidFilter("ENST00000155093"),
+                                  column = "tx_name")
+    checkTrue(any(colnames(mcols(fUTRs[[1]])) == "tx_name"))
+
     do.plot <- FALSE
-    fUTRs <- fiveUTRsByTranscript(DB, filter=list(SeqnameFilter("Y"), SeqstrandFilter("+")))
-    tUTRs <- threeUTRsByTranscript(DB, filter=list(SeqnameFilter("Y"), SeqstrandFilter("+")))
-    cds <- cdsBy(DB, "tx", filter=list(SeqnameFilter("Y"), SeqstrandFilter("+")))
+    fUTRs <- fiveUTRsByTranscript(DB, filter = list(SeqnameFilter("Y"),
+                                                    SeqstrandFilter("+")))
+    tUTRs <- threeUTRsByTranscript(DB, filter = list(SeqnameFilter("Y"),
+                                                     SeqstrandFilter("+")))
+    cds <- cdsBy(DB, "tx", filter = list(SeqnameFilter("Y"),
+                                         SeqstrandFilter("+")))
     ## Check a TX:
     tx <- names(fUTRs)[1]
-    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx=tx, do.plot=do.plot)
+    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx = tx,
+                  do.plot = do.plot)
     tx <- names(fUTRs)[2]
-    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx=tx, do.plot=do.plot)
+    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx = tx,
+                  do.plot = do.plot)
     tx <- names(fUTRs)[3]
-    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx=tx, do.plot=do.plot)
+    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx = tx,
+                  do.plot = do.plot)
 
     ## Reverse strand
-    fUTRs <- fiveUTRsByTranscript(DB, filter=list(SeqnameFilter("Y"), SeqstrandFilter("-")))
-    tUTRs <- threeUTRsByTranscript(DB, filter=list(SeqnameFilter("Y"), SeqstrandFilter("-")))
-    cds <- cdsBy(DB, "tx", filter=list(SeqnameFilter("Y"), SeqstrandFilter("-")))
+    fUTRs <- fiveUTRsByTranscript(DB, filter = list(SeqnameFilter("Y"),
+                                                    SeqstrandFilter("-")))
+    tUTRs <- threeUTRsByTranscript(DB, filter = list(SeqnameFilter("Y"),
+                                                     SeqstrandFilter("-")))
+    cds <- cdsBy(DB, "tx", filter = list(SeqnameFilter("Y"),
+                                         SeqstrandFilter("-")))
     ## Check a TX:
     tx <- names(fUTRs)[1]
-    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx=tx, do.plot=do.plot)
+    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx = tx,
+                  do.plot = do.plot)
     tx <- names(fUTRs)[2]
-    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx=tx, do.plot=do.plot)
+    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx = tx,
+                  do.plot = do.plot)
     tx <- names(fUTRs)[3]
-    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx=tx, do.plot=do.plot)
+    checkGeneUTRs(fUTRs[[tx]], tUTRs[[tx]], cds[[tx]], tx = tx,
+                  do.plot = do.plot)
+}
+
+## The "test_UTRs" has a very poor performance with the RSQLite 1.0.9011
+## release candidate. Here we want to evaluate the performance.
+dontrun_test_UTRs_performance <- function() {
+    system.time(fUTRs <- fiveUTRsByTranscript(DB,
+                                              filter = list(SeqnameFilter("Y"),
+                                                            SeqstrandFilter("+")),
+                                              column = "tx_name")
+                )
+    ## 6.4 secs.
+    system.time(fUTRs <- fiveUTRsByTranscript(DB,
+                                              filter = list(SeqnameFilter("Y"),
+                                                            SeqstrandFilter("+"))))
+    ## 6.4 secs.
+    system.time(tUTRs <- threeUTRsByTranscript(DB,
+                                               filter = list(SeqnameFilter("Y"),
+                                                             SeqstrandFilter("+"))))
+    ## 6.3 secs.
+    system.time(cds <- cdsBy(DB, "tx", filter = list(SeqnameFilter("Y"),
+                                                     SeqstrandFilter("+"))))
+    ## 6.3 secs.
+    system.time(fUTRs <- fiveUTRsByTranscript(DB,
+                                              filter = list(SeqnameFilter("Y"),
+                                                            SeqstrandFilter("-"))))
+    ## 6.4 secs.
+    system.time(tUTRs <- threeUTRsByTranscript(DB,
+                                               filter = list(SeqnameFilter("Y"),
+                                                             SeqstrandFilter("-"))))
+    ## 6.6 secs.
+    system.time(cds <- cdsBy(DB, "tx", filter = list(SeqnameFilter("Y"),
+                                                     SeqstrandFilter("-"))))
+    ## 6.3 secs.
 }
 
 checkGeneUTRs <- function(f, t, c, tx, do.plot=FALSE){
